@@ -35,7 +35,7 @@ int LMS7_LimeNET_micro::Init()
         {0x002D, 0x0641}, {0x0086, 0x4101}, {0x0087, 0x5555}, {0x0088, 0x03F0},
         {0x0089, 0x1078}, {0x008B, 0x2100}, {0x008C, 0x267B}, {0x00A1, 0x656A},
         {0x00A6, 0x0009}, {0x00A7, 0x8A8A}, {0x00A9, 0x8000}, {0x00AC, 0x2000},
-        {0x0105, 0x0011}, {0x0108, 0x118C}, {0x0109, 0x6100}, {0x010A, 0x1F4C},
+        {0x0105, 0x0011}, {0x0108, 0x218C}, {0x0109, 0x6100}, {0x010A, 0x1F4C},
         {0x010B, 0x0001}, {0x010C, 0x8865}, {0x010E, 0x0000}, {0x010F, 0x3142},
         {0x0110, 0x2B14}, {0x0111, 0x0000}, {0x0112, 0x942E}, {0x0113, 0x03C2},
         {0x0114, 0x00D0}, {0x0117, 0x1230}, {0x0119, 0x18D2}, {0x011C, 0x8941},
@@ -52,6 +52,10 @@ int LMS7_LimeNET_micro::Init()
     lms->Modify_SPI_Reg_bits(LMS7param(MAC), 1);
     for (auto i : initVals)
         lms->SPI_write(i.adr, i.val, true);
+
+    if(lms->CalibrateTxGain(0,nullptr) != 0)
+        return -1;
+
     lms->EnableChannel(true, false);
 
     lms->Modify_SPI_Reg_bits(LMS7param(MAC), 2);
@@ -63,6 +67,18 @@ int LMS7_LimeNET_micro::Init()
 
     lms->Modify_SPI_Reg_bits(LMS7param(MAC), 1);
 
+    bool auto_path[2] = {auto_tx_path,auto_rx_path};
+    auto_tx_path = false;
+    auto_rx_path = false;
+    
+    if(SetFrequency(true, 0, GetFrequency(true, 0))!=0)
+        return -1;
+    if(SetFrequency(false, 0, GetFrequency(false, 0))!=0)
+        return -1;
+
+    auto_tx_path = auto_path[0];
+    auto_rx_path = auto_path[1];
+    
     if (SetRate(1e6, 16)!=0)
         return -1;
 
