@@ -480,17 +480,9 @@ bool ConnectionFT601::WaitForReading(int contextHandle, unsigned int timeout_ms)
             if (dwRet == WAIT_OBJECT_0)
                 return 1;
 #else
-        auto t1 = chrono::high_resolution_clock::now();
-        auto t2 = t1;
-
+        //blocking not to waste CPU
         std::unique_lock<std::mutex> lck(contexts[contextHandle].transferLock);
-        while(contexts[contextHandle].done.load() == false && std::chrono::duration_cast<std::chrono::milliseconds>(t2 - t1).count() < timeout_ms)
-        {
-            //blocking not to waste CPU
-            contexts[contextHandle].cv.wait_for(lck, chrono::milliseconds(timeout_ms));
-            t2 = chrono::high_resolution_clock::now();
-        }
-        return contexts[contextHandle].done.load() == true;
+        return contexts[contextHandle].cv.wait_for(lck, chrono::milliseconds(timeout_ms), [&](){return contexts[contextHandle].done.load();});
 #endif
     }
     return true;  //there is nothing to wait for (signal wait finished)
@@ -633,16 +625,9 @@ bool ConnectionFT601::WaitForSending(int contextHandle, unsigned int timeout_ms)
             if (dwRet == WAIT_OBJECT_0)
                 return 1;
 #else
-        auto t1 = chrono::high_resolution_clock::now();
-        auto t2 = t1;
+        //blocking not to waste CPU
         std::unique_lock<std::mutex> lck(contextsToSend[contextHandle].transferLock);
-        while(contextsToSend[contextHandle].done.load() == false && std::chrono::duration_cast<std::chrono::milliseconds>(t2 - t1).count() < timeout_ms)
-        {
-            //blocking not to waste CPU
-            contextsToSend[contextHandle].cv.wait_for(lck, chrono::milliseconds(timeout_ms));
-            t2 = chrono::high_resolution_clock::now();
-        }
-        return contextsToSend[contextHandle].done == true;
+        return contextsToSend[contextHandle].cv.wait_for(lck, chrono::milliseconds(timeout_ms), [&](){return contextsToSend[contextHandle].done.load();});
 #endif
     }
     return true; //there is nothing to wait for (signal wait finished)
@@ -746,6 +731,8 @@ int ConnectionFT601::ResetStreamBuffers()
 
 int ConnectionFT601::ProgramWrite(const char *data_src, size_t length, int prog_mode, int device, ProgrammingCallback callback)
 {
+    int ret;
+
     if (device != LMS64CProtocol::FPGA)
     {
         lime::error("Unsupported programming target");
@@ -759,27 +746,40 @@ int ConnectionFT601::ProgramWrite(const char *data_src, size_t length, int prog_
 
     if (prog_mode == 2)
         return LMS64CProtocol::ProgramWrite(data_src, length, prog_mode, device, callback);
-    if (GetFPGAInfo().gatewareVersion != 0)
+
+    FPGAinfo fpgainfo = GetFPGAInfo();
+
+    if (fpgainfo.hwVersion < 3)
     {
+        // LimeSDR-Mini v1.X
+        if (fpgainfo.gatewareVersion != 0)
+        {
+            LMS64CProtocol::ProgramWrite(nullptr, 0, 2, 2, nullptr);
+            std::this_thread::sleep_for(std::chrono::milliseconds(2000));
+        }
+        const int sizeUFM = 0x8000;
+        const int sizeCFM0 = 0x42000;
+        const int startUFM = 0x1000;
+        const int startCFM0 = 0x4B000;
+
+        if (length != startCFM0 + sizeCFM0)
+        {
+            lime::error("Invalid image file");
+            return -1;
+        }
+        std::vector<char> buffer(sizeUFM + sizeCFM0);
+        memcpy(buffer.data(), data_src + startUFM, sizeUFM);
+        memcpy(buffer.data() + sizeUFM, data_src + startCFM0, sizeCFM0);
+
+        ret = LMS64CProtocol::ProgramWrite(buffer.data(), buffer.size(), prog_mode, device, callback);
         LMS64CProtocol::ProgramWrite(nullptr, 0, 2, 2, nullptr);
-        std::this_thread::sleep_for(std::chrono::milliseconds(2000));
     }
-    const int sizeUFM = 0x8000;
-    const int sizeCFM0 = 0x42000;
-    const int startUFM = 0x1000;
-    const int startCFM0 = 0x4B000;
-
-    if (length != startCFM0 + sizeCFM0)
+    else
     {
-        lime::error("Invalid image file");
-        return -1;
+        // LimeSDR-Mini v2.X
+        ret = LMS64CProtocol::ProgramWrite(data_src, length, prog_mode, device, callback);
+        LMS64CProtocol::ProgramWrite(nullptr, 0, 2, 2, nullptr);
     }
-    std::vector<char> buffer(sizeUFM + sizeCFM0);
-    memcpy(buffer.data(), data_src + startUFM, sizeUFM);
-    memcpy(buffer.data() + sizeUFM, data_src + startCFM0, sizeCFM0);
-
-    int ret = LMS64CProtocol::ProgramWrite(buffer.data(), buffer.size(), prog_mode,  device, callback);
-    LMS64CProtocol::ProgramWrite(nullptr, 0, 2, 2, nullptr);
 
     return ret;
 }
